﻿{*******************************************************}
{                                                       }
{       Database Clase Base Class Unit                  }
{                                                       }
{       Copyright (c) 2016 Hideaki Tominaga             }
{                                                       }
{*******************************************************}
unit uClassBaseClass;

//{$DEFINE USE_BDE}
{$DEFINE USE_IBX}
{$DEFINE USE_DBGO}
//{$DEFINE USE_DBX}
//{$DEFINE USE_FIREDAC}

interface

uses
  Classes, SysUtils,
  {$IFDEF USE_BDE}
  DBTables,
  {$ENDIF}
  {$IFDEF USE_IBX}
  IBX.IBQuery,
  {$ENDIF}
  {$IFDEF USE_DBGO}
  ADODB,
  {$ENDIF}
  {$IFDEF USE_DBX}
  SqlExpr,
  {$ENDIF}
  {$IFDEF USE_FIREDAC}
  FireDAC.Comp.Client,
  {$ENDIF}
  Rtti, TypInfo, DB;

type
  TClassBaseClass = class (TComponent)
  private
    FDataSet: TDataSet;
  public
    // Methids
    procedure GetFields;
    procedure SetParams;
    // Properties
    property DataSet: TDataSet read FDataSet write FDataSet;
  end;

implementation

{ TClassBaseClass }

procedure TClassBaseClass.GetFields;
var
  Ctx: TRTTIConText;
  ParamName, FieldName, ClassName: string;
  Field: TRttiField;
  i: Integer;
  Param: TField;
begin
  Ctx := TRttiContext.Create;
  try
    ClassName := Self.UnitName + '.' + Self.ClassName;
    for i:=0 to FDataSet.FieldCount-1 do
      begin
        ParamName := FDataSet.Fields[i].FieldName;
        Param := FDataSet.FieldByName(ParamName);
        FieldName := 'F' + ParamName;
        Field := Ctx.FindType(ClassName).GetField(FieldName);
        if Field = nil then
          Continue;
        case Param.DataType of
          ftBoolean:
            Field.SetValue(Self, Param.AsBoolean);
          ftShortint,
          ftByte,
          ftSmallint,
          ftWord,
          ftInteger,
          ftLongWord:
            Field.SetValue(Self, Param.AsInteger);
          ftLargeint:
            Field.SetValue(Self, Param.AsLargeInt);
          ftCurrency:
            Field.SetValue(Self, Param.AsCurrency);
          ftSingle:
            Field.SetValue(Self, Param.AsSingle);
          ftExtended:
            Field.SetValue(Self, Param.AsExtended);
          ftFloat:
            Field.SetValue(Self, Param.AsFloat);
          ftDate,
          ftTime,
          ftDateTime,
          ftTimeStamp:
            Field.SetValue(Self, Param.AsDateTime);
          ftFmtMemo,
          ftMemo,
          ftWideMemo,
          ftWideString,
          ftString:
            Field.SetValue(Self, Param.AsString);
        else
          // ftUnknown
        end;
      end;
  finally
    Ctx.Free;
  end;
end;

procedure TClassBaseClass.SetParams;
type
  TDataSetType = (dstUnknown, dstBDEQuery, dstIBXQuery, dstDBGOQuery, dstDBXQuery, dstFireDACQuery);
var
  i: Integer;
  ParamName, FieldName, ClassName: string;
  Field: TRttiField;
  KeyParams: Boolean;
  Ctx: TRTTIConText;
  Param: TParam;
  Parameter: TParameter;
  ParamCnt: Integer;
  DataSetType: TDataSetType;
begin
  Ctx := TRttiContext.Create;
  try
    ClassName := Self.UnitName + '.' + Self.ClassName;
    DataSetType := dstUnknown;
    {$IFDEF USE_BDE}
    if (FDataSet is TQuery) then
      DataSetType := dstBDEQuery;
    {$ENDIF}
    {$IFDEF USE_IBX}
    if (FDataSet is TIBQuery) then
      DataSetType := dstIBXQuery;
    {$ENDIF}
    {$IFDEF USE_DBGO}
    if (FDataSet is TADOQuery) then
      DataSetType := dstDBGOQuery;
    {$ENDIF}
    {$IFDEF USE_DBX}
    if (FDataSet is TSQLQuery) then
      DataSetType := dstDBXQuery;
    {$ENDIF}
    {$IFDEF USE_FIREDAC}
    if (FDataSet is TFDQuery) then
      DataSetType := dstFireDACQuery;
    {$ENDIF}

    case DataSetType of
      {$IFDEF USE_BDE}
      dstBDEQuery:
        ParamCnt := (FDataSet as TQuery).ParamCount;
      {$ENDIF}
      {$IFDEF USE_IBX}
      dstIBXQuery:
        ParamCnt := (FDataSet as TIBQuery).ParamCount;
      {$ENDIF}
      {$IFDEF USE_ADO}
      dstDBGOQuery:
        ParamCnt := (FDataSet as TADOQuery).Parameters.Count;
      {$ENDIF}
      {$IFDEF USE_DBX}
      dstDBXQuery:
        ParamCnt := (FDataSet as TSQLQuery).Params.Count;
      {$ENDIF}
      {$IFDEF USE_FIREDAC}
      dstFireDACQuery:
        ParamCnt := (FDataSet as TFDQuery).ParamCount;
      {$ENDIF}
    else
      ParamCnt := 0;
    end;

    for i:=0 to ParamCnt-1 do
      begin
        case DataSetType of
          {$IFDEF USE_BDE}
          dstBDEQuery:
            ParamName := (FDataSet as TQuery).Params[i].Name;
          {$ENDIF}
          {$IFDEF USE_IBX}
          dstIBXQuery:
            ParamName := (FDataSet as TIBQuery).Params[i].Name;
          {$ENDIF}
          {$IFDEF USE_ADO}
          dstDBGOQuery:
            ParamName := (FDataSet as TADOQuery).Parameters[i].Name;
          {$ENDIF}
          {$IFDEF USE_DBX}
          dstDBXQuery:
            ParamName := (FDataSet as TSQLQuery).Params[i].Name;
          {$ENDIF}
          {$IFDEF USE_FIREDAC}
          dstFireDACQuery:
            ParamName := (FDataSet as TFDQuery).Params[i].Name;
          {$ENDIF}
        else
          ParamName := '';
        end;
        FieldName := ParamName;
        KeyParams := ParamName[1] = '_';
        if KeyParams then
          FieldName := Copy(FieldName, 2, Length(FieldName));
        FieldName := 'F' + FieldName;

        Field := Ctx.FindType(ClassName).GetField(FieldName);
        case DataSetType of
          // BDE, IBX, DBX, FireDac
          dstBDEQuery,
          dstIBXQuery,
          dstDBXQuery,
          dstFireDACQuery:
            begin
              case DataSetType of
                {$IFDEF USE_BDE}
                dstBDEQuery:
                  Param := (FDataSet as TQuery).ParamByName(ParamName);
                {$ENDIF}
                {$IFDEF USE_IBX}
                dstIBXQuery:
                  Param := (FDataSet as TIBQuery).ParamByName(ParamName);
                {$ENDIF}
                {$IFDEF USE_DBX}
                dstDBXQuery:
                  Param := (FDataSet as TSQLQuery).ParamByName(ParamName);
                {$ENDIF}
                {$IFDEF USE_FIREDAC}
                dstFireDACQuery:
                  Param := (FDataSet as TFDQuery).ParamByName(ParamName);
                {$ENDIF}
              else
                Param := nil;
              end;
              if Param <> nil then
                begin
                  if LowerCase(Field.FieldType.Name) = LowerCase('Integer') then
                    begin
                      Param.DataType  := ftInteger;
                      Param.AsInteger := Field.GetValue(Self).AsInteger;
                    end
                  else if LowerCase(Field.FieldType.Name) = LowerCase('SmallInt') then
                    begin
                      Param.DataType   := ftSmallint;
                      Param.AsSmallInt := Field.GetValue(Self).AsInteger;
                    end
                  else if LowerCase(Field.FieldType.Name) = LowerCase('Int64') then
                    begin
                      Param.DataType := ftLargeint;
                      Param.AsLargeInt  := Field.GetValue(Self).AsInt64;
                    end
                  else if LowerCase(Field.FieldType.Name) = LowerCase('Single') then
                    begin
                      Param.DataType := ftSingle;
                      Param.AsSingle  := Field.GetValue(Self).AsExtended;
                    end
                  else if LowerCase(Field.FieldType.Name) = LowerCase('Double') then
                    begin
                      Param.DataType := ftFloat;
                      Param.AsFloat  := Field.GetValue(Self).AsExtended;
                    end
                  else if LowerCase(Field.FieldType.Name) = LowerCase('Extended') then
                    begin
                      Param.DataType := ftExtended;
                      Param.AsFloat  := Field.GetValue(Self).AsExtended;
                    end
                  else if LowerCase(Field.FieldType.Name) = LowerCase('Currency') then
                    begin
                      Param.DataType   := ftCurrency;
                      Param.AsCurrency := Field.GetValue(Self).AsCurrency;
                    end
                  else if LowerCase(Field.FieldType.Name) = LowerCase('TDateTime') then
                    begin
                      Param.DataType   := ftDateTime;
                      Param.AsDateTime := Field.GetValue(Self).AsExtended;
                    end
                  else
                    begin
                      Param.DataType  := ftString;
                      Param.AsString  := Field.GetValue(Self).AsString;
                    end;
                end;
            end;
          // dbGo
          dstDBGOQuery:
            begin
              Parameter := (FDataSet as TADOQuery).Parameters.ParamByName(ParamName);
              if LowerCase(Field.FieldType.Name) = LowerCase('Integer') then
                begin
                  Parameter.DataType  := ftInteger;
                  Parameter.Value := Field.GetValue(Self).AsInteger;
                end
              else if LowerCase(Field.FieldType.Name) = LowerCase('SmallInt') then
                begin
                  Parameter.DataType   := ftSmallint;
                  Parameter.Value := Field.GetValue(Self).AsInteger;
                end
              else if LowerCase(Field.FieldType.Name) = LowerCase('Int64') then
                begin
                  Parameter.DataType := ftLargeint;
                  Parameter.Value := Field.GetValue(Self).AsInt64;
                end
              else if LowerCase(Field.FieldType.Name) = LowerCase('Single') then
                begin
                  Parameter.DataType := ftSingle;
                  Parameter.Value := Field.GetValue(Self).AsExtended;
                end
              else if LowerCase(Field.FieldType.Name) = LowerCase('Double') then
                begin
                  Parameter.DataType := ftFloat;
                  Parameter.Value := Field.GetValue(Self).AsExtended;
                end
              else if LowerCase(Field.FieldType.Name) = LowerCase('Extended') then
                begin
                  Parameter.DataType := ftExtended;
                  Parameter.Value := Field.GetValue(Self).AsExtended;
                end
              else if LowerCase(Field.FieldType.Name) = LowerCase('Currency') then
                begin
                  Parameter.DataType   := ftCurrency;
                  Parameter.Value := Field.GetValue(Self).AsCurrency;
                end
              else if LowerCase(Field.FieldType.Name) = LowerCase('TDateTime') then
                begin
                  Parameter.DataType   := ftDateTime;
                  Parameter.Value := Field.GetValue(Self).AsExtended;
                end
              else
                begin
                  Parameter.DataType  := ftString;
                  Parameter.Value  := Field.GetValue(Self).AsString;
                end;
            end;
        end;
      end;
  finally
    Ctx.Free;
  end;
end;
end.

